'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fusionCore = require('fusion-core');
var fusionTokens = require('fusion-tokens');
var fusionPluginApollo = require('fusion-plugin-apollo');
var apolloClient = require('apollo-client');
var apolloLinkHttp = require('apollo-link-http');
var apolloLink = require('apollo-link');
var apolloLinkSchema = require('apollo-link-schema');
var apolloCacheInmemory = require('apollo-cache-inmemory');
var Cookies = require('js-cookie');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
// Fixed By: https://github.com/benmosher/eslint-plugin-import/issues/975#issuecomment-348807796
// eslint-disable-next-line
var GetApolloClientCacheToken = fusionCore.createToken('GetApolloClientCacheToken');
var ApolloClientCredentialsToken = fusionCore.createToken('ApolloClientCredentialsToken');
var ApolloClientDefaultOptionsToken = fusionCore.createToken('ApolloClientDefaultOptionsToken');
var GetApolloClientLinksToken = fusionCore.createToken('GetApolloClientLinksToken');
var ApolloClientAuthKeyToken = fusionCore.createToken('ApolloClientAuthKeyToken');
var ApolloClientResolversToken = fusionCore.createToken('ApolloClientResolversToken');

function Container() {}

var ApolloClientPlugin = fusionCore.createPlugin({
  deps: {
    getCache: GetApolloClientCacheToken.optional,
    endpoint: fusionPluginApollo.GraphQLEndpointToken.optional,
    fetch: fusionTokens.FetchToken,
    includeCredentials: ApolloClientCredentialsToken.optional,
    authKey: ApolloClientAuthKeyToken.optional,
    apolloContext: fusionPluginApollo.ApolloContextToken.optional,
    getApolloLinks: GetApolloClientLinksToken.optional,
    schema: fusionPluginApollo.GraphQLSchemaToken.optional,
    resolvers: ApolloClientResolversToken.optional,
    defaultOptions: ApolloClientDefaultOptionsToken.optional
  },
  provides: function provides(_ref) {
    var _ref$getCache = _ref.getCache,
        getCache = _ref$getCache === void 0 ? function (ctx) {
      return new apolloCacheInmemory.InMemoryCache();
    } : _ref$getCache,
        _ref$endpoint = _ref.endpoint,
        endpoint = _ref$endpoint === void 0 ? '/graphql' : _ref$endpoint,
        fetch = _ref.fetch,
        _ref$authKey = _ref.authKey,
        authKey = _ref$authKey === void 0 ? 'token' : _ref$authKey,
        _ref$includeCredentia = _ref.includeCredentials,
        includeCredentials = _ref$includeCredentia === void 0 ? 'same-origin' : _ref$includeCredentia,
        _ref$apolloContext = _ref.apolloContext,
        apolloContext = _ref$apolloContext === void 0 ? function (ctx) {
      return ctx;
    } : _ref$apolloContext,
        getApolloLinks = _ref.getApolloLinks,
        schema = _ref.schema,
        resolvers = _ref.resolvers,
        defaultOptions = _ref.defaultOptions;

    function getClient(ctx, initialState) {
      var cache = getCache(ctx);

      var getBrowserProps = function getBrowserProps() {
        return Cookies.get(authKey);
      };

      var connectionLink = schema && false ? new apolloLinkSchema.SchemaLink({
        schema: schema,
        context: typeof apolloContext === 'function' ? apolloContext(ctx) : apolloContext
      }) : new apolloLinkHttp.HttpLink({
        uri: endpoint,
        credentials: includeCredentials,
        fetch: fetch
      });
      var token = getBrowserProps();
      var authMiddleware = new apolloLink.ApolloLink(function (operation, forward) {
        if (token) {
          operation.setContext({
            headers: {
              authorization: "Bearer " + token
            }
          });
        }

        return forward(operation);
      });
      var links = getApolloLinks ? getApolloLinks([authMiddleware, connectionLink], ctx) : [authMiddleware, connectionLink];
      var client = new apolloClient.ApolloClient({
        ssrMode: false,
        connectToDevTools: true && process.env.NODE_ENV !== "production",
        link: apolloLink.from(links),
        cache: cache.restore(initialState),
        resolvers: resolvers,
        defaultOptions: defaultOptions
      });
      return client;
    }

    return function (ctx, initialState) {
      if (ctx.memoized.has(Container)) {
        return ctx.memoized.get(Container);
      }

      var client = getClient(ctx, initialState);
      ctx.memoized.set(Container, client);
      return client;
    };
  }
});

exports.GetApolloClientCacheToken = GetApolloClientCacheToken;
exports.ApolloClientCredentialsToken = ApolloClientCredentialsToken;
exports.ApolloClientDefaultOptionsToken = ApolloClientDefaultOptionsToken;
exports.GetApolloClientLinksToken = GetApolloClientLinksToken;
exports.ApolloClientAuthKeyToken = ApolloClientAuthKeyToken;
exports.ApolloClientResolversToken = ApolloClientResolversToken;
exports.default = ApolloClientPlugin;
//# sourceMappingURL=browser.es5.js.map
