'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fusionCore = require('fusion-core');
var fusionTokens = require('fusion-tokens');
var fusionPluginApollo = require('fusion-plugin-apollo');
var apolloClient = require('apollo-client');
var apolloLinkHttp = require('apollo-link-http');
var apolloLink = require('apollo-link');
var apolloLinkSchema = require('apollo-link-schema');
var apolloCacheInmemory = require('apollo-cache-inmemory');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
// Fixed By: https://github.com/benmosher/eslint-plugin-import/issues/975#issuecomment-348807796
// eslint-disable-next-line
const GetApolloClientCacheToken = fusionCore.createToken('GetApolloClientCacheToken');
const ApolloClientCredentialsToken = fusionCore.createToken('ApolloClientCredentialsToken');
const ApolloClientDefaultOptionsToken = fusionCore.createToken('ApolloClientDefaultOptionsToken');
const GetApolloClientLinksToken = fusionCore.createToken('GetApolloClientLinksToken');
const ApolloClientAuthKeyToken = fusionCore.createToken('ApolloClientAuthKeyToken');
const ApolloClientResolversToken = fusionCore.createToken('ApolloClientResolversToken');

function Container() {}

const ApolloClientPlugin = fusionCore.createPlugin({
  deps: {
    getCache: GetApolloClientCacheToken.optional,
    endpoint: fusionPluginApollo.GraphQLEndpointToken.optional,
    fetch: fusionTokens.FetchToken,
    includeCredentials: ApolloClientCredentialsToken.optional,
    authKey: ApolloClientAuthKeyToken.optional,
    apolloContext: fusionPluginApollo.ApolloContextToken.optional,
    getApolloLinks: GetApolloClientLinksToken.optional,
    schema: fusionPluginApollo.GraphQLSchemaToken.optional,
    resolvers: ApolloClientResolversToken.optional,
    defaultOptions: ApolloClientDefaultOptionsToken.optional
  },

  provides({
    getCache = ctx => new apolloCacheInmemory.InMemoryCache(),
    endpoint = '/graphql',
    fetch,
    authKey = 'token',
    includeCredentials = 'same-origin',
    apolloContext = ctx => ctx,
    getApolloLinks,
    schema,
    resolvers,
    defaultOptions
  }) {
    function getClient(ctx, initialState) {
      const cache = getCache(ctx);

      const getServerProps = () => {
        return ctx && ctx.cookies.get(authKey);
      };

      const connectionLink = schema && true ? new apolloLinkSchema.SchemaLink({
        schema,
        context: typeof apolloContext === 'function' ? apolloContext(ctx) : apolloContext
      }) : new apolloLinkHttp.HttpLink({
        uri: endpoint,
        credentials: includeCredentials,
        fetch
      });
      const token = getServerProps();
      const authMiddleware = new apolloLink.ApolloLink((operation, forward) => {
        if (token) {
          operation.setContext({
            headers: {
              authorization: `Bearer ${token}`
            }
          });
        }

        return forward(operation);
      });
      const links = getApolloLinks ? getApolloLinks([authMiddleware, connectionLink], ctx) : [authMiddleware, connectionLink];
      const client = new apolloClient.ApolloClient({
        ssrMode: true,
        connectToDevTools: false && process.env.NODE_ENV !== "production",
        link: apolloLink.from(links),
        cache: cache.restore(initialState),
        resolvers,
        defaultOptions
      });
      return client;
    }

    return (ctx, initialState) => {
      if (ctx.memoized.has(Container)) {
        return ctx.memoized.get(Container);
      }

      const client = getClient(ctx, initialState);
      ctx.memoized.set(Container, client);
      return client;
    };
  }

});

exports.GetApolloClientCacheToken = GetApolloClientCacheToken;
exports.ApolloClientCredentialsToken = ApolloClientCredentialsToken;
exports.ApolloClientDefaultOptionsToken = ApolloClientDefaultOptionsToken;
exports.GetApolloClientLinksToken = GetApolloClientLinksToken;
exports.ApolloClientAuthKeyToken = ApolloClientAuthKeyToken;
exports.ApolloClientResolversToken = ApolloClientResolversToken;
exports.default = ApolloClientPlugin;
//# sourceMappingURL=index.js.map
